"""Argument parsing."""

from argparse import ArgumentParser, Namespace
from pathlib import Path

from homeinfotools.os import CACHE_DIR


__all__ = ['get_args']


CACHE_FILE = CACHE_DIR / 'sysquery.cache'


def get_args() -> Namespace:
    """Returns parsed CLI arguments."""

    parser = ArgumentParser(description='Retrieve and filter systems.')
    parser.add_argument(
        '-U', '--user', metavar='account', help='the HIS user account'
    )
    parser.add_argument(
        '-i', '--id', nargs='+', type=int, metavar='id',
        help='filter by system IDs'
    )
    parser.add_argument(
        '-o', '--os', nargs='+', metavar='operating system',
        help='filter by operating systems')
    parser.add_argument(
        '--sn', nargs='+', metavar='serial number',
        help='filter by serial numbers'
    )
    parser.add_argument(
        '-D', '--deployment', nargs='+', type=int, metavar='deployment',
        help='filter by deployments'
    )
    parser.add_argument(
        '-C', '--customer', nargs='+', type=str.casefold,
        metavar='customer', help='filter by customers'
    )
    parser.add_argument(
        '-t', '--type', nargs='+', metavar='type', help='filter by types'
    )
    parser.add_argument(
        '-s', '--street', nargs='+', type=str.casefold, metavar='street',
        help='filter by streets'
    )
    parser.add_argument(
        '-H', '--house-number', nargs='+', type=str.casefold,
        metavar='house number', help='filter by house numbers'
    )
    parser.add_argument(
        '-z', '--zip-code', nargs='+', type=str.casefold,
        metavar='zip code', help='filter by zip codes'
    )
    parser.add_argument(
        '-c', '--city', nargs='+', type=str.casefold, metavar='city',
        help='filter by cities'
    )
    parser.add_argument(
        '-v', '--verbose', action='store_true', help='enable verbose mode'
    )
    parser.add_argument(
        '-d', '--debug', action='store_true', help='enable debug mode'
    )
    parser.add_argument(
        '-f', '--refresh', action='store_true',
        help='force refreshing of cache'
    )
    parser.add_argument(
        '--cache-file', type=Path, default=CACHE_FILE, metavar='file',
        help='use the given file as cache'
    )
    parser.add_argument(
        '--cache-time', type=int, default=24, metavar='hours',
        help='sets the caching time in hours'
    )
    return parser.parse_args()



"""Argument parsing."""

from argparse import ArgumentParser, Namespace
from pathlib import Path


__all__ = ['get_args']


def get_args() -> Namespace:
    """Parses the command line arguments."""

    parser = ArgumentParser(description='OpenVPN configuration retrieval.')
    parser.add_argument('system', type=int, help='the system ID')
    parser.add_argument(
        '-d', '--debug', action='store_true', help='enable debug mode'
    )
    parser.add_argument(
        '-f', '--file', type=Path, metavar='filename', help='output file'
    )
    parser.add_argument('-m', '--model', metavar='model', help='Set model')
    parser.add_argument(
        '-o', '--operating-system', metavar='operating_system', help='Set OS'
    )
    parser.add_argument(
        '-p', '--pubkey', metavar='public_key', help='Set WireGuard pubkey'
    )
    parser.add_argument(
        '-s', '--serial-number', metavar='serial_number',
        help='Set serial number'
    )
    parser.add_argument(
        '-u', '--user', metavar='his_account', help='HIS user name'
    )
    parser.add_argument(
        '-w', '--windows', action='store_true',
        help='package for MS Windows systems'
    )
    return parser.parse_args()



"""HIS SSO API."""

from getpass import getpass
from sys import exit

from homeinfotools.logging import LOGGER


__all__ = ['update_credentials']


def update_credentials(
        account: str | None,
        passwd: str | None = None
) -> tuple[str, str]:
    """Reads the credentials for a HIS account."""

    if not account:
        try:
            account = input('User name: ')
        except (EOFError, KeyboardInterrupt):
            print()
            LOGGER.error('Aborted by user.')
            exit(1)

    if not passwd:
        try:
            passwd = getpass('Password: ')
        except (EOFError, KeyboardInterrupt):
            print()
            LOGGER.error('Aborted by user.')
            exit(1)

    return account, passwd



"""SSH command."""

from pathlib import Path
from subprocess import CalledProcessError, check_call

from homeinfotools.os import SSH, RSYNC


__all__ = ['ssh', 'rsync']


HOSTNAME = '{}.terminals.homeinfo.intra'
SSH_OPTIONS = [
    'LogLevel=error',
    'UserKnownHostsFile=/dev/null',
    'StrictHostKeyChecking=no',
    'ConnectTimeout=5'
]
TRUE = '/usr/bin/true'
USERS = {'homeinfo', 'root'}
HostPath = Path | tuple[int, Path]


def ssh(
        system: int | None,
        *command: str,
        user: str | None = None,
        no_stdin: bool = False
) -> list[str]:
    """Modifies the specified command to
    run via SSH on the specified system.
    """

    cmd = [SSH]

    if no_stdin:
        cmd.append('-n')

    for option in SSH_OPTIONS:
        cmd.append('-o')
        cmd.append(option)

    if user is None:
        user = get_ssh_user(system)

    if system is not None:
        hostname = HOSTNAME.format(system)

        if user is not None:
            hostname = f'{user}@{hostname}'

        cmd.append(hostname)

    if command:
        cmd.append(' '.join(command))

    return cmd


def rsync(
        src: HostPath,
        dst: HostPath,
        *,
        all: bool = True,
        update: bool = True,
        verbose: bool = True
) -> list[str]:
    """Returns the respective rsync command."""

    cmd = [RSYNC, '-e', ' '.join(ssh(None))]

    if all:
        cmd.append('-a')

    if update:
        cmd.append('-u')

    if verbose:
        cmd.append('-v')

    return cmd + [get_remote_path(src), get_remote_path(dst)]


def get_remote_path(path: HostPath) -> str:
    """Returns a host path."""

    try:
        system, path = path
    except TypeError:
        return path

    return HOSTNAME.format(system) + f':{path}'


def get_ssh_user(system: int) -> str | None:
    """Returns the SSH user."""

    for user in USERS:
        try:
            check_call(ssh(system, TRUE, user=user))
        except CalledProcessError:
            continue

        return user

    return None
